/* 
 * CSE142, Spring 1999
 * Homework 4
 * Traffic animation
 * Bill Dougherty
 * Section VA

  STARTER PROGRAM

 *
 *
 */

/********************************************************************

  Stuff I added:

  Most notably, Godzilla.  At least, that's what it's supposed to
  look like and, believe me, what you see represents the ultimate
  stretch of my artistic talent.  You move Godzilla around by
  clicking somewhere below the horizon.  Any cars in his way get 
  crushed.  If he's standing on the road and a car comes along, he
  stomps it.  Smashed cars will continue to move, inexplicably so
  except by the fact that it would take too much (more) time to
  have cars stopped all over the place.
  
  Also, the closer he gets the bigger he looks.  The intensity of
  perspective and the base size are both scalable by changing #defined constants.
  	
  Cars appear randomly from one side or the other depending on which
  direction traffic is moving.  Once off the screen they are gone
  forever.  The maximum number of vehicles is scalable by a #define
  constant.
  
********************************************************************/

#include "gp142.h"
#include <stdio.h>
#include <math.h>

#include <time.h>
	/* 
	* <time.h> is included so the current time can be accessed
	* to initialize the random number generator.  If you don't
	* use random numbers (and you don't have to) you can either
	* ignore this #include or delete it.
	*/

/* Logical constants */
#define TRUE    		1
#define FALSE   		0

/* 
 * For many of the GP142 functions, a line width of
 * zero means to fill the shape being drawn.  The
 * constant FILL is used in function calls when a solid
 * shape is desired.
 */
#define FILL 0


/*
 * Symbolic constants.  Change as needed.
 */

/* Menu buttons: */

/* Top-left coordinates of button palette */
#define MENU_TOP		(GP142_YMAX - 5)
#define MENU_LEFT		(-GP142_XMAX + 5)

/* Height and width of buttons in pixels */
#define BUTTON_HEIGHT  	25
#define BUTTON_WIDTH 	75

/* Number of buttons */
#define NUM_BUTTONS 	6


/* Possible locations of mouse clicks: */

/*   Mouse click in some button */
#define BUTTON_CLICK 1

/*   Mouse click elsewhere */
#define OTHER_CLICK 0


/* Coordinates of objects in the drawing */

/* y-coordinate of the top of the ground in the landscape */
#define HORIZON (- 2 * (GP142_YMAX / 10))

/* Vertical center and width of highway */
#define HIGHWAY_CENTER ((-GP142_YMAX + HORIZON) / 3)

#define HIGHWAY_WIDTH  10


/* Maximum number of vehicles that can appear in the animation */
#define MAX_VEHICLES 10

/* Height and width of a single car */
#define CAR_HEIGHT 24
#define CAR_WIDTH  40

/* Car status */
#define CAR_OK 0
#define CAR_SMASHED 1
#define CAR_ABOUT_SMASHED 2				/* smash after very next cycle */

/* Probability per cycle of a new car */
#define CAR_PROB_NEW .1

/* Godzilla behavior values */
#define GZ_STANDING 0
#define GZ_WALKING_NF_UP 1				/* walking, near foot up */
#define GZ_WALKING_FF_UP 2				/* walking, far foot up */

/* Godzilla initial conditions */
#define GZ_XSTART 0
#define GZ_YSTART -GP142_XMAX + 100
#define GZ_ACT_START GZ_STANDING

/* Godzilla attributes */
#define GZ_CYCLE_DELAY 2
#define GZ_SIZE_SCALE 10
#define GZ_PERSPECTIVE (double)(1.3 / (HIGHWAY_CENTER - (-GP142_XMAX)))

#define GZ_COL1 FOREST_GREEN
#define GZ_COL2 GREEN
#define GZ_COL3 RED
#define GZ_COL4 BLACK

/* Where Godzilla smashes cars.  Relative to current Godzilla */
/* coordinates.  X is scaled in smash_cars */
#define GZ_SMASH_FORWARD_X 2
#define GZ_SMASH_BACKWARD_X -6
#define GZ_SMASH_UP_Y 15
#define GZ_SMASH_DOWN_Y -10

/*****************************************************************************

     FUNCTION PROTOTYPES

*****************************************************************************/

/* Initialize random number generator.  [Ignore or		*/
/* remove if your program doesn't use random numbers.]	*/
void initialize_random_numbers();

/* Draw the background, including the sun at coordinates sun_x and sun_y */
void draw_landscape(int sun_x, int sun_y);
void draw_highway(int highway_center);

/* Draw the buttons */
void draw_buttons();

/* Return kind of mouse click (in a button, etc.) */
int  classify_mouse_click(int mouse_x, int mouse_y);

/* Process button click at location x, y.  Update car speed,  */
/* direction, color, and whether traffic is moving as needed. */
void handle_button(int x, int y, int *traffic_moving, int *car_speed,
				   int *car_direction, int *car_color);

/* Update sun coordinates sun_x, sun_y for one tick */
void move_sun(int *sun_x, int *sun_y);

/* Update all vehicle x, y coordinates, given current x, y */
/* location, direction (+1 or -1), and speed in pixels per tick. */
void move_traffic(int x[], int y[], int *direction, int speed,
				  int* numcars, int car_status[]);

/* Draw all vehicles given their current x, y coordinates and color */
void draw_traffic(int x[], int y[], int status[], int color, int numcars);

/* Move Godzilla */
void move_godzilla(int* gz_x, int* gz_y, int* gz_action, int* gz_xtarg,
				   int* gz_ytarg, int speed, int* gz_facing,
				   int* gz_cycle_count, double* gz_size);

/* Draw Godzilla */
void draw_godzilla(int x, int y, int action, int facing, double size);

/* Draw Godzilla with near foot raised */
void draw_gz_nfup(int x, int y, int facing, double size);

/* Draw Godzilla with far foot raised */
void draw_gz_ffup(int x, int y, int facing, double size);

/* Draw Godzilla standing */
void draw_gz_standing(int x, int y, int facing, double size);

/* Given new godzilla and car positions, update car status to smashed */
/* if appropriate */
void smash_cars(int x[], int y[], int status[], int gz_x, int gz_y,
				int numcars, int facing, int *gz_action,
				int* gz_cycle_count);

/* Generate new car with probability < 1 */
void new_traffic(int x[], int y[], int status[], int car_direction,
				 int* numcars);


/* Draw a single car of the given color with horizontal */
/* center x and bottom y*/
void draw_car(int x, int y, int color);

/* Draw a smashed car of the given color */
void draw_smashed_car(int x, int y, int color);

/* Display error message if internal error is detected. */
void something_is_wrong (void);


/*****************************************************************************

     MAIN

*****************************************************************************/

/*
 * Main program contents:
 *   - Major program variables that persist throughout execution
 *   - Event loop that processes GP142 events until the user quits,
 *     and updates variables and redraws the window as needed.
 */
int main(void)
{
	/* Event handling and buttons: */
	int  quit;				/* = "user has selected quit"				*/
	int  event;				/* the most recent GP142 event				*/
	char key_pressed;		/* last keyboard character from key event	*/
	int  mouse_x, mouse_y;	/* x and y coordinates of latest mouse event*/
	int  click_kind;		/* kind of mouse click (button, etc.)		*/

	/* Information about the sun: */
	int sun_x, sun_y;			/* screen x and y coordinates */

	/* Information about vehicles in the animation:							*/
	/* [Only one car now - will need to be changed to add more]				*/
	int car_x[MAX_VEHICLES], car_y[MAX_VEHICLES];		/* current position of the car					*/
	int car_status[MAX_VEHICLES];  /*car smashed, on fire, or okay?			*/
	int car_direction;		/* current direction: +1=right, -1=left			*/
	int car_color;			/* color of the car								*/
	int car_speed;			/* speed in pixels per tick when traffic moving	*/	
							/*   note--godzilla moves 2x as fast			*/
	int traffic_moving;		/* = "traffic is currently moving"				*/
	int numcars;			/* number of currently existing cars */

	/*Godzilla stuff*/
	int gz_x, gz_y;			/* Godzilla x and y coordinates */
	int gz_action;			/* Godzilla current behavior */
	int gz_xtarg, gz_ytarg;	/* movement target coordinates*/
	int gz_facing;			/* +1 facing right, -1 facing left */
	int gz_cycle_count;		/* don't move legs to fast to see */
	double gz_size;			/* size varies with perspective */

	initialize_random_numbers();

	/* Initialize graphics package, clear the window,	*/
	/* turn logging off, and start animation.			*/
	GP142_open();
	GP142_logging(LOG_OFF);
	GP142_clear();
	GP142_animate(ANI_RUN);

	/* set initial location of the sun */
	sun_x = GP142_XMAX - 50;
	sun_y = GP142_YMAX - 35;

	/* set initial conditions for the cars */
	
		numcars = 0;
		car_direction = +1;
		car_speed = 5;
		car_color = RED;
		traffic_moving = TRUE;

	/* set Godzilla initial conditions */
	gz_x = GZ_XSTART;
	gz_y = GZ_YSTART;
	gz_xtarg = gz_x;
	gz_ytarg = gz_y;
	gz_action = GZ_ACT_START;
	gz_facing = +1;
	gz_cycle_count = 0;
	gz_size = 1 + (HIGHWAY_CENTER - GZ_YSTART) * GZ_PERSPECTIVE;

	/* Draw initial landscape, buttons, and traffic */
	draw_landscape(sun_x,sun_y);
	draw_buttons( );
	draw_traffic(car_x, car_y, car_status, car_color, numcars);
	draw_godzilla(GZ_XSTART, GZ_YSTART, GZ_ACT_START, gz_facing,
					gz_size);

	/*
	 * Main event loop:
	 * ---- ----- -----
	 * Wait for the next user action, decode it, and call an
	 * appropriate function to handle it.  Repeat until the
	 * user selects quit.
	 */
	quit = FALSE;
	while (!quit) {
		
		/* get next event */
		event = GP142_await_event(&mouse_x, &mouse_y, &key_pressed);
		
		/* perform appropriate action depending on kind of event */
		switch (event) {
		case GP142_QUIT:
			/* Quit selected */
			quit = TRUE;
			break;
			
		case GP142_KBD:
			/* Key pressed.  Not used now.  If the program should		*/
			/* react to keyboard events, insert something appropriate	*/
			/* here.  Note that scanf CANNOT be used with GP142.		*/
			break;
			
		case GP142_MOUSE:
			/* Mouse click.  Determine if in a menu button	*/
			/* or elsewhere and process accordingly.		*/
			click_kind = classify_mouse_click(mouse_x, mouse_y);
			if (click_kind == BUTTON_CLICK) {
				handle_button(mouse_x, mouse_y, &traffic_moving, &car_speed,
								&car_direction, &car_color);
			} else {
				/* move godzilla target to the new location */
				if (mouse_y <= HORIZON) {
					gz_xtarg = mouse_x;
					gz_ytarg = mouse_y;
				}
			}
			break;
			
		case GP142_PERIODIC:
			/* Timer interval event.  Update everything	*/
			/* that needs to change as time advances.	*/
			move_sun(&sun_x, &sun_y);
			if (traffic_moving) {
				move_traffic(car_x, car_y, &car_direction, car_speed,
							&numcars, car_status);
				new_traffic(car_x, car_y, car_status, car_direction,
							&numcars);
			}
			move_godzilla(&gz_x, &gz_y, &gz_action, &gz_xtarg,
						&gz_ytarg, car_speed, &gz_facing,
						&gz_cycle_count, &gz_size);
			smash_cars(car_x, car_y, car_status, gz_x, gz_y, numcars,
						gz_facing, &gz_action, &gz_cycle_count);
			break;
			
		default:
			/* unknown event */
			break;
			
		}  /*end switch */
		
		/* Redraw everything */
		draw_landscape(sun_x, sun_y);
		draw_buttons();
		draw_traffic(car_x, car_y, car_status, car_color, numcars);
		draw_godzilla(gz_x, gz_y, gz_action, gz_facing, gz_size);
		
	}  /* end event loop */
	
	
	/* Termination: shut down graphics window and exit */
	GP142_close();
	return 0;
}  /*end main */


/*****************************************************************************

     BACKGROUND LANDSCAPE

*****************************************************************************/


/* Draw the background, including the sun at coordinates sun_x and sun_y */
void draw_landscape(int sun_x, int sun_y)
{
	int text_offset = -GP142_XMAX + 15;	/* horizontal offset of text */
	int text_size   = 12;				/* size of text font */

	/* Draw the sky */
	GP142_rectangleXY(ICE_BLUE, -GP142_XMAX, -GP142_YMAX,
		GP142_XMAX, GP142_YMAX, FILL);
	
	/* Draw the sun */
	GP142_circleXY(YELLOW, sun_x, sun_y , 20);
	
	/* Draw the ground, highway, and text legend */
	GP142_rectangleXY(BROWN, -GP142_XMAX, -GP142_YMAX,
		GP142_XMAX, HORIZON, FILL);
	
	draw_highway(HIGHWAY_CENTER);

	GP142_printfXY (BLACK, text_offset, -GP142_YMAX+30, text_size + 4,
		"The Amazing Perspecta-Godzilla,\"No Rights Deserved\".");
	GP142_printfXY (BLACK, text_offset, -GP142_YMAX+10, text_size,
		"Clock a spot to move him there!  (\"There\" should be somewhere on the road for maximum effect.)");
	
	/*Draw some mountains */
	GP142_triangleXY(MED_GRAY,
		-GP142_XMAX, HORIZON,
		GP142_XMAX, HORIZON,
		0, (4 * GP142_YMAX)/5 ,
		FILL);
	
	GP142_triangleXY(OLIVE,
		-( 5 * GP142_XMAX/4), HORIZON,
		(GP142_XMAX)/3  , HORIZON,
		-(3* GP142_XMAX)/4, (2 * GP142_YMAX)/3 ,
		FILL);
	
	GP142_triangleXY(LT_GRAY,
		-GP142_XMAX / 3 , HORIZON,
		GP142_XMAX, HORIZON,
		GP142_XMAX, (2 * GP142_YMAX)/3 ,
		FILL);
	
} /*end draw_landscape */

/*draw a highway*/
void draw_highway(int highway_center) {
	
	GP142_rectangleXY(WHITE, 
					-GP142_XMAX, highway_center - HIGHWAY_WIDTH,
					 GP142_XMAX, highway_center + HIGHWAY_WIDTH,
					 FILL);
	GP142_rectangleXY(BLACK,
					-GP142_XMAX, highway_center,
					 GP142_XMAX, highway_center,
					 FILL);
}


/*****************************************************************************

     BUTTONS AND MOUSE CLICKS

*****************************************************************************/


/* Draws the buttons */
void draw_buttons( )
{
	int i;
	int text_offset = MENU_LEFT+10;	/* offset of button text in button */
	int font_size = 12;				/* size of button text */
	
	/* draw rectangles for each button */
	for(i = 0; i < NUM_BUTTONS; i++) {
		GP142_rectangleXY(BLACK,
			MENU_LEFT,
			MENU_TOP - i * BUTTON_HEIGHT,
			MENU_LEFT + BUTTON_WIDTH,
			MENU_TOP - (i + 1)*BUTTON_HEIGHT, 1);
	}
	
	/* display button text */
	i = MENU_TOP - (BUTTON_HEIGHT/2) - (font_size/2);
	GP142_printfXY(FOREST_GREEN, text_offset, i, font_size, "Go");	
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(RED, text_offset, i, font_size, "Stop");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "Faster");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "Slower");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "Reverse");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "New Color");
	
}

/* Classify mouse click at (mouse_x,mouse_y) and return kind */
int  classify_mouse_click(int mouse_x, int mouse_y)
{
	if ((mouse_x > MENU_LEFT) &&
		(mouse_x < MENU_LEFT + BUTTON_WIDTH) &&
		(mouse_y < MENU_TOP) &&
		(mouse_y > (MENU_TOP - NUM_BUTTONS*BUTTON_HEIGHT)))
		return BUTTON_CLICK;
	else
		return OTHER_CLICK;
}



/* Process button click at location x, y.  Update car speed,	*/
/* direction, color, and whether traffic is moving as needed.	*/
void handle_button(int x, int y, int *traffic_moving, int *car_speed,
				   int *car_direction, int *car_color)
{
	int buttonNum;		/* number of selected button */

	/* Complain if x coordinate couldn't be inside a button */
	if (x < MENU_LEFT || x > MENU_LEFT + BUTTON_WIDTH) {
		something_is_wrong();
		return;
	}
	
	/* Convert y coordinate to button number from top. */
	buttonNum = (MENU_TOP - y)/BUTTON_HEIGHT;	 
	
	switch(buttonNum) {
	case 0:
		/* go */
		*traffic_moving = TRUE;
		break;
	case 1:
		/* stop */
		*traffic_moving = FALSE;
		break;
		
	case 2:
		/* speed up */
		*car_speed = *car_speed + 1;
		break;
	case 3:
		/* slow down */
		*car_speed = *car_speed - 1;
		if (*car_speed < 0)
			*car_speed = 0;
		break;
		
	case 4:
		/* reverse */
		*car_direction = - *car_direction;
		break;
	case 5:
		/* change car color */
		*car_color = (*car_color+1) % MAX_COLORS;
		break;
	default:
		/*Bad news if this is ever reached! */
		something_is_wrong();
		break;
	}
}



/*****************************************************************************

     TRAFFIC AND OTHER ANIMATED OBJECTS

*****************************************************************************/


/* Update sun coordinates sun_x, sun_y by one tick */
void move_sun(int *sun_x, int *sun_y) {
	/* Move the sun to the left.  If it reaches the edge, move it */
	/* to the other side and down a bit. */
	(*sun_x) --;
	if (*sun_x < -GP142_XMAX -30) {
		*sun_x = - *sun_x;
		(*sun_y) --;
	}
}


/* Update all vehicle x, y coordinates, given current x, y */
/* location, direction (+1 or -1), and speed in pixels per tick, */
void move_traffic(int car_x[], int car_y[], int *direction, int speed,
				  int* numcars, int car_status[]) {
	/* Note: If there is more than one vehicle, this should */
	/* be updated to adjust all vehicle coordinates */

	int trial_x;		/* new value for x coordinate */
	int carnum;			/* loop counter */
	int loop;		/* another loop counter */

	/* move vehicle.  If it moves off the edge, delete it */
	
	for (carnum = (*numcars) - 1; carnum >= 0; carnum--) {
		trial_x = car_x[carnum] + (*direction * speed);
		if (trial_x > GP142_XMAX + CAR_WIDTH/2) {
			(*numcars)--;
		}
		else {
			if (trial_x < -GP142_XMAX - CAR_WIDTH/2) {
				for (loop = carnum; loop <= *numcars - 2; loop++) {
					car_x[loop] = car_x[loop + 1];
					car_y[loop] = car_y[loop + 1];
					car_status[loop] = car_status[loop + 1];
				}
				(*numcars)--;
			}
			else car_x[carnum] = trial_x;
		}				
	}
}

/* Smash cars where godzilla is standing */
void smash_cars(int x[], int y[], int status[], int gz_x, int gz_y,
				int numcars, int facing, int* gz_action,
				int* gz_cycle_count) {
	
	int carnum;				/* car loop counter */
	int gsx;				/* Godzilla scale */
	int oldstatus;			/* status before possible stomp */

	gsx = GZ_SIZE_SCALE;

	if (facing == +1) {
		for (carnum = 0; carnum <= numcars - 1; carnum++) {
			oldstatus = status[carnum];
			if (  (status[carnum] == CAR_ABOUT_SMASHED) ||
				(status[carnum] == CAR_SMASHED) ) {
				status[carnum] = CAR_SMASHED;
			}
			else if ( (x[carnum] <= gz_x + gsx * GZ_SMASH_FORWARD_X) &&
					(x[carnum] >= gz_x + gsx * GZ_SMASH_BACKWARD_X) &&
					(y[carnum] <= gz_y + GZ_SMASH_UP_Y) &&
					(y[carnum] >= gz_y + GZ_SMASH_DOWN_Y) ) {
						status[carnum] = CAR_ABOUT_SMASHED;
						if (  (*gz_action == GZ_STANDING) &&
							(oldstatus != CAR_SMASHED) ) {
							*gz_action = GZ_WALKING_NF_UP;
							*gz_cycle_count = GZ_CYCLE_DELAY;
				
						}
			}
	
		}
	}
	else {
		for (carnum = 0; carnum <= numcars - 1; carnum++) {
			oldstatus = status[carnum];
			if ( (status[carnum] == CAR_ABOUT_SMASHED) ||
				(status[carnum] == CAR_SMASHED)  ) {
				status[carnum] = CAR_SMASHED;
			}
			else if ( (x[carnum] >= gz_x - gsx * GZ_SMASH_FORWARD_X) &&
				 (x[carnum] <= gz_x - gsx * GZ_SMASH_BACKWARD_X) &&
				(y[carnum] <= gz_y + GZ_SMASH_UP_Y) &&
				(y[carnum] >= gz_y + GZ_SMASH_DOWN_Y) ) {
				status[carnum] = CAR_ABOUT_SMASHED;
					if (  (*gz_action == GZ_STANDING) &&
						(oldstatus != CAR_SMASHED)  ) {
						*gz_action = GZ_WALKING_NF_UP;
						*gz_cycle_count = GZ_CYCLE_DELAY;
					}
			}
		}
	}
}

/* Draw all vehicles given their current x, y coordinates and color */
void draw_traffic(int x[], int y[], int status[], int color, int numcars) {
	/* Note: needs to be changed if there is more than */
	/* one vehicle */

	int carnum;		/*loop counter*/
	
	for (carnum = 0; carnum <= numcars - 1; carnum++) {
		switch(status[carnum]) {
		case CAR_OK:
			draw_car(x[carnum],y[carnum],color);
			break;
		case CAR_ABOUT_SMASHED:
			draw_car(x[carnum],y[carnum],color);
			break;
		case CAR_SMASHED:
			draw_smashed_car(x[carnum],y[carnum],color);
			break;
		default:
			draw_smashed_car(x[carnum],y[carnum],color);
			break;
		}
	}
}



/* Draw a single car of the given color with horizontal */
/* center x and bottom y*/
void draw_car(int x, int y, int color) {
	GP142_rectangleXY(color, x - CAR_WIDTH/2, y + CAR_HEIGHT/4,
						x + CAR_WIDTH/2, y + CAR_HEIGHT*3/4, FILL);
	GP142_rectangleXY(color, x - CAR_WIDTH/6, y + CAR_HEIGHT*3/4,
						x + CAR_WIDTH/6, y + CAR_HEIGHT, FILL);
	GP142_circleXY(BLACK, x - CAR_WIDTH/4, y + CAR_HEIGHT/4, CAR_HEIGHT/4);
	GP142_circleXY(BLACK, x + CAR_WIDTH/4, y + CAR_HEIGHT/4, CAR_HEIGHT/4);
}


/* Draw a single smashed car of the given color with horizontal */
/* center x and bottom y*/
void draw_smashed_car(int x, int y, int color) {
	GP142_rectangleXY(color, x - CAR_WIDTH/2, y, 
						x + CAR_WIDTH/2, y + CAR_HEIGHT/4, FILL);
	GP142_rectangleXY(BLACK, x - CAR_WIDTH/4 - CAR_HEIGHT/4, y,
							 x - CAR_WIDTH/4 + CAR_HEIGHT/4,
							 y + CAR_HEIGHT/4, FILL);
	GP142_rectangleXY(BLACK, x + CAR_WIDTH/4 - CAR_HEIGHT/4, y,
							 x + CAR_WIDTH/4 + CAR_HEIGHT/4,
							 y + CAR_HEIGHT/4, FILL);
	GP142_triangleXY(color, 
					 x - CAR_WIDTH/4 + CAR_HEIGHT/4, y + CAR_HEIGHT/4,
					 x - CAR_WIDTH / 2, y + CAR_HEIGHT/4,
					 x - CAR_WIDTH / 2, y + CAR_HEIGHT*5/16,
					 FILL);
	GP142_triangleXY(color,
					 x + CAR_WIDTH/4, y + CAR_HEIGHT/4,
					 x, y + CAR_HEIGHT/4,
					 x + CAR_WIDTH*2/3, y + CAR_HEIGHT/2,
					 FILL);


}



/* Put in new traffic */
void new_traffic(int x[], int y[], int status[], int direction,
				 int* numcars) {

	int carnum;			/* loop counter */

	if  (
		  ( rand() < RAND_MAX * CAR_PROB_NEW ) &&
		  ( *numcars < MAX_VEHICLES )		&&
		  (  
	 		( direction==+1 && x[0] > -GP142_XMAX + CAR_WIDTH ) ||
 		    ( direction==-1 && x[(*numcars)-1] < GP142_XMAX - CAR_WIDTH ) || 
 			( *numcars == 0 )
     	  )
		)  {
							
		 
		 
		 if (direction == +1) {
			/* shift arrays right to make room for new (enters left) */
			for (carnum = (*numcars) - 1 ; carnum >= 0; carnum--) {
				x[carnum+1] = x[carnum];
				y[carnum+1] = y[carnum];
				status[carnum+1] = status[carnum];
			}
			x[0] = -GP142_XMAX - CAR_WIDTH/2;
			y[0] = HIGHWAY_CENTER;
			status[0] = CAR_OK;
		}
			/* add car to the end of arrays (entering on right) */
		else {
			x[*numcars] = GP142_XMAX + CAR_WIDTH/2;
			y[*numcars] = HIGHWAY_CENTER;
			status[*numcars] = CAR_OK;
		}		
		(*numcars)++;
	}


}

/*****************************************************************************

     GODZILLA FUNCTIONS

*****************************************************************************/

/* Move Godzilla */
void move_godzilla(int* gz_x, int* gz_y, int* gz_action, 
					int* gz_xtarg, int* gz_ytarg, int speed,
					int* gz_facing, int* gz_cycle_count,
					double* gz_size) {
	int trial_x, trial_y;		/* trial godzilla coordinates */

	/*if right of target*/
	if (*gz_x > *gz_xtarg) {
		trial_x = *gz_x - 2 * speed;
		if (trial_x < *gz_xtarg)
			trial_x = *gz_xtarg;
		else *gz_facing = -1;
	}
	/*if left of target*/
	else {
		trial_x = *gz_x + 2 * speed;
		if (trial_x > *gz_xtarg)
			trial_x = *gz_xtarg;
		else *gz_facing = +1;
	}
	
	/*Now if above target*/
	if (*gz_y > *gz_ytarg) {
		trial_y = *gz_y - 2*speed;
		if (trial_y < *gz_ytarg)
			trial_y = *gz_ytarg;
	}
	/*but if below target*/
	else {
		trial_y = *gz_y + 2*speed;
		if (trial_y > *gz_ytarg)
			trial_y = *gz_ytarg;
	}
	
	
	/* change to new behavior -- alternate foot up or standing */
	if ((trial_x == *gz_xtarg) && (trial_y == *gz_ytarg)) {
		*gz_action = GZ_STANDING;
		*gz_cycle_count = 0;
	}
	else {
		(*gz_cycle_count)++;
		if ( *gz_cycle_count >= GZ_CYCLE_DELAY) {
			*gz_cycle_count = 0;
			if (*gz_action == GZ_WALKING_FF_UP)
				*gz_action = GZ_WALKING_NF_UP;
			else *gz_action = GZ_WALKING_FF_UP;
		}
	}
	
	
	*gz_x = trial_x;
	*gz_y = trial_y;
	*gz_size = 1 + (HIGHWAY_CENTER - *gz_y) * GZ_PERSPECTIVE;

}

/* Draw Godzilla with horizontal center x and vertical bottom y */
void draw_godzilla(int x, int y, int action, int facing,
				   double size) {

	switch (action) {
	case GZ_WALKING_NF_UP:
		draw_gz_nfup(x, y, facing, size);
		break;
	case GZ_WALKING_FF_UP:
		draw_gz_ffup(x, y, facing, size);
		break;
	case GZ_STANDING:
		draw_gz_standing(x, y, facing, size);
		break;
	default:
		GP142_rectangleXY(BLACK, x - 50, y, x + 50, y + 100, FILL);
		break;
	}

}

/* Draw Godzilla with horizontal center x, base y, near foot up */
void draw_gz_nfup(int x, int y, int facing, double size) {

	int gsx;				/* GZ step size, x */
	int gsy;				/* and y */
	
	gsx = (int)(GZ_SIZE_SCALE * facing * size);
	gsy = (int)(GZ_SIZE_SCALE * size);

	/* draw far foot (down) */
	GP142_triangleXY( GZ_COL4,
					x + 1 *gsx, y + 1 *gsy,
					x - 1 *gsx, y + 3 *gsy,
					x - 5 *gsx,	y + 1 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL4,
					x + 1 *gsx, y + 9 *gsy,
					x - 5 *gsx, y + 1 *gsy,
					x - 1 *gsx,	y + 3 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL4,
					x + 1 *gsx, y + 9 *gsy,
					x - 3 *gsx, y + 5 *gsy,
					x - 5 *gsx,	y + 1 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL4,
					x + 1 *gsx, y + 9 *gsy,
					x - 3 *gsx, y + 11*gsy,
					x - 6 *gsx,	y + 7 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL4,
					x + 1 *gsx, y + 9 *gsy,
					x - 6 *gsx, y + 7 *gsy,
					x - 3 *gsx,	y + 5 *gsy,
					FILL);
		
	/* draw head, body, tail */
	GP142_triangleXY( GZ_COL1,
					x + 10*gsx, y + 20*gsy,
					x + 4 *gsx, y + 23*gsy,
					x,			y + 20*gsy,
					FILL);
	GP142_triangleXY( GZ_COL1,
					x + 10*gsx, y + 20*gsy,
					x + 7 *gsx, y + 19*gsy,
					x,			y + 20*gsy,
					FILL);
	GP142_triangleXY( GZ_COL1,
					x + 7 *gsx, y + 19*gsy,
					x + 8 *gsx, y + 16*gsy,
					x + 4 *gsx,	y + 18*gsy,
					FILL);
	GP142_triangleXY( GZ_COL1,
					x + 7 *gsx, y + 19*gsy,
					x + 0 *gsx, y + 20*gsy,
					x + 4 *gsx,	y + 18*gsy,
					FILL);
	GP142_triangleXY( GZ_COL1,
					x + 4 *gsx, y + 18*gsy,
					x + 0 *gsx, y + 20*gsy,
					x + 0 *gsx,	y + 2 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL1,
					x + 0 *gsx, y + 2 *gsy,
					x + 0 *gsx, y + 20*gsy,
					x - 6 *gsx,	y + 8 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL1,
					x + 0 *gsx, y + 2 *gsy,
					x - 6 *gsx, y + 8 *gsy,
					x - 6 *gsx,	y + 4 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL1,
					x - 6 *gsx, y + 4 *gsy,
					x - 6 *gsx, y + 8 *gsy,
					x - 16*gsx,	y + 4 *gsy,
					FILL);

	/* draw forarms */
	GP142_triangleXY( GZ_COL2,
					x + 6 *gsx, y + 12*gsy,
					x + 6 *gsx, y + 13*gsy,
					x + 2 *gsx,	y + 15*gsy,
					FILL);
	GP142_triangleXY( GZ_COL2,
					x + 6 *gsx, y + 12*gsy,
					x + 2 *gsx, y + 15*gsy,
					x + 2 *gsx,	y + 14*gsy,
					FILL);

	/* draw near foot and leg up*/
	GP142_triangleXY( GZ_COL2,
					x + 6 *gsx, y + 4 *gsy,
					x + 3 *gsx, y + 6 *gsy,
					x + 0 *gsx,	y + 4 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL2,
					x + 4 *gsx, y + 10*gsy,
					x + 0 *gsx, y + 4 *gsy,
					x + 3 *gsx,	y + 6 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL2,
					x + 4 *gsx, y + 10*gsy,
					x + 0 *gsx, y + 6 *gsy,
					x + 0 *gsx,	y + 4 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL2,
					x + 4 *gsx, y + 10*gsy,
					x - 4 *gsx, y + 8 *gsy,
					x + 0 *gsx,	y + 6 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL2,
					x + 0 *gsx, y + 6 *gsy,
					x - 4 *gsx, y + 8 *gsy,
					x - 4 *gsx,	y + 4 *gsy,
					FILL);

	/* draw gleaming GZ_COL3, most likely red, eye */
	GP142_triangleXY( GZ_COL3,
					x + 6 *gsx, y + 21*gsy,
					x + 4 *gsx, y + 22*gsy,
					x + 5 *gsx,	y + 21*gsy,
					FILL);

}

void draw_gz_ffup(int x, int y, int facing, double size) {

	int gsx;				/* GZ scale size, x */
	int gsy;				/* and y */
	
	gsx = (int)(GZ_SIZE_SCALE * facing * size);
	gsy = (int)(GZ_SIZE_SCALE * size);
	
	/* draw far foot (up)*/
	GP142_triangleXY( GZ_COL4,
					x + 7 *gsx, y + 5 *gsy,
					x + 4 *gsx, y + 7 *gsy,
					x + 1 *gsx,	y + 5 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL4,
					x + 5 *gsx, y + 11*gsy,
					x + 1 *gsx, y + 5 *gsy,
					x + 4 *gsx,	y + 7 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL4,
					x + 5 *gsx, y + 11*gsy,
					x + 1 *gsx, y + 7 *gsy,
					x + 1 *gsx,	y + 5 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL4,
					x + 5 *gsx, y + 11*gsy,
					x - 3 *gsx, y + 9 *gsy,
					x + 1 *gsx,	y + 7 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL4,
					x + 1 *gsx, y + 7 *gsy,
					x - 3 *gsx, y + 9 *gsy,
					x - 3 *gsx,	y + 5 *gsy,
					FILL);

	/* draw head, body, tail */
	GP142_triangleXY( GZ_COL1,
					x + 10*gsx, y + 20*gsy,
					x + 4 *gsx, y + 23*gsy,
					x,			y + 20*gsy,
					FILL);
	GP142_triangleXY( GZ_COL1,
					x + 10*gsx, y + 20*gsy,
					x + 7 *gsx, y + 19*gsy,
					x,			y + 20*gsy,
					FILL);
	GP142_triangleXY( GZ_COL1,
					x + 7 *gsx, y + 19*gsy,
					x + 8 *gsx, y + 16*gsy,
					x + 4 *gsx,	y + 18*gsy,
					FILL);
	GP142_triangleXY( GZ_COL1,
					x + 7 *gsx, y + 19*gsy,
					x + 0 *gsx, y + 20*gsy,
					x + 4 *gsx,	y + 18*gsy,
					FILL);
	GP142_triangleXY( GZ_COL1,
					x + 4 *gsx, y + 18*gsy,
					x + 0 *gsx, y + 20*gsy,
					x + 0 *gsx,	y + 2 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL1,
					x + 0 *gsx, y + 2 *gsy,
					x + 0 *gsx, y + 20*gsy,
					x - 6 *gsx,	y + 8 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL1,
					x + 0 *gsx, y + 2 *gsy,
					x - 6 *gsx, y + 8 *gsy,
					x - 6 *gsx,	y + 4 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL1,
					x - 6 *gsx, y + 4 *gsy,
					x - 6 *gsx, y + 8 *gsy,
					x - 16*gsx,	y + 4 *gsy,
					FILL);

	/* draw forarms */
	GP142_triangleXY( GZ_COL2,
					x + 6 *gsx, y + 12*gsy,
					x + 6 *gsx, y + 13*gsy,
					x + 2 *gsx,	y + 15*gsy,
					FILL);
	GP142_triangleXY( GZ_COL2,
					x + 6 *gsx, y + 12*gsy,
					x + 2 *gsx, y + 15*gsy,
					x + 2 *gsx,	y + 14*gsy,
					FILL);

	/* draw near foot (down) */
	GP142_triangleXY( GZ_COL2,
					x + 0 *gsx, y + 0 *gsy,
					x - 2 *gsx, y + 2 *gsy,
					x - 6 *gsx,	y + 0 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL2,
					x + 0 *gsx, y + 8 *gsy,
					x - 6 *gsx, y + 0 *gsy,
					x - 2 *gsx,	y + 2 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL2,
					x + 0 *gsx, y + 8 *gsy,
					x - 4 *gsx, y + 4 *gsy,
					x - 6 *gsx,	y + 0 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL2,
					x + 0 *gsx, y + 8 *gsy,
					x - 4 *gsx, y + 10*gsy,
					x - 7 *gsx,	y + 6 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL2,
					x + 0 *gsx, y + 8 *gsy,
					x - 7 *gsx, y + 6 *gsy,
					x - 4 *gsx,	y + 4 *gsy,
					FILL);

	/* draw gleaming GZ_COL3, most likely red, eye */
	GP142_triangleXY( GZ_COL3,
					x + 6 *gsx, y + 21*gsy,
					x + 4 *gsx, y + 22*gsy,
					x + 5 *gsx,	y + 21*gsy,
					FILL);

}

void draw_gz_standing(int x, int y, int facing, double size) {

	int gsx;				/* GZ scale size, x */
	int gsy;				/* and y */
	
	gsx = (int)(GZ_SIZE_SCALE * facing * size);
	gsy = (int)(GZ_SIZE_SCALE * size);

	/* draw far foot (down) */
	GP142_triangleXY( GZ_COL4,
					x + 1 *gsx, y + 1 *gsy,
					x - 1 *gsx, y + 3 *gsy,
					x - 5 *gsx,	y + 1 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL4,
					x + 1 *gsx, y + 9 *gsy,
					x - 5 *gsx, y + 1 *gsy,
					x - 1 *gsx,	y + 3 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL4,
					x + 1 *gsx, y + 9 *gsy,
					x - 3 *gsx, y + 5 *gsy,
					x - 5 *gsx,	y + 1 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL4,
					x + 1 *gsx, y + 9 *gsy,
					x - 3 *gsx, y + 11*gsy,
					x - 6 *gsx,	y + 7 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL4,
					x + 1 *gsx, y + 9 *gsy,
					x - 6 *gsx, y + 7 *gsy,
					x - 3 *gsx,	y + 5 *gsy,
					FILL);
	
	
	/* draw head, body, tail */
	GP142_triangleXY( GZ_COL1,
					x + 10*gsx, y + 20*gsy,
					x + 4 *gsx, y + 23*gsy,
					x,			y + 20*gsy,
					FILL);
	GP142_triangleXY( GZ_COL1,
					x + 10*gsx, y + 20*gsy,
					x + 7 *gsx, y + 19*gsy,
					x,			y + 20*gsy,
					FILL);
	GP142_triangleXY( GZ_COL1,
					x + 7 *gsx, y + 19*gsy,
					x + 8 *gsx, y + 16*gsy,
					x + 4 *gsx,	y + 18*gsy,
					FILL);
	GP142_triangleXY( GZ_COL1,
					x + 7 *gsx, y + 19*gsy,
					x + 0 *gsx, y + 20*gsy,
					x + 4 *gsx,	y + 18*gsy,
					FILL);
	GP142_triangleXY( GZ_COL1,
					x + 4 *gsx, y + 18*gsy,
					x + 0 *gsx, y + 20*gsy,
					x + 0 *gsx,	y + 2 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL1,
					x + 0 *gsx, y + 2 *gsy,
					x + 0 *gsx, y + 20*gsy,
					x - 6 *gsx,	y + 8 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL1,
					x + 0 *gsx, y + 2 *gsy,
					x - 6 *gsx, y + 8 *gsy,
					x - 6 *gsx,	y + 4 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL1,
					x - 6 *gsx, y + 4 *gsy,
					x - 6 *gsx, y + 8 *gsy,
					x - 16*gsx,	y + 4 *gsy,
					FILL);

	/* draw forarms */
	GP142_triangleXY( GZ_COL2,
					x + 6 *gsx, y + 12*gsy,
					x + 6 *gsx, y + 13*gsy,
					x + 2 *gsx,	y + 15*gsy,
					FILL);
	GP142_triangleXY( GZ_COL2,
					x + 6 *gsx, y + 12*gsy,
					x + 2 *gsx, y + 15*gsy,
					x + 2 *gsx,	y + 14*gsy,
					FILL);

	/* draw near foot (down) */
	GP142_triangleXY( GZ_COL2,
					x + 0 *gsx, y + 0 *gsy,
					x - 2 *gsx, y + 2 *gsy,
					x - 6 *gsx,	y + 0 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL2,
					x + 0 *gsx, y + 8 *gsy,
					x - 6 *gsx, y + 0 *gsy,
					x - 2 *gsx,	y + 2 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL2,
					x + 0 *gsx, y + 8 *gsy,
					x - 4 *gsx, y + 4 *gsy,
					x - 6 *gsx,	y + 0 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL2,
					x + 0 *gsx, y + 8 *gsy,
					x - 4 *gsx, y + 10*gsy,
					x - 7 *gsx,	y + 6 *gsy,
					FILL);
	GP142_triangleXY( GZ_COL2,
					x + 0 *gsx, y + 8 *gsy,
					x - 7 *gsx, y + 6 *gsy,
					x - 4 *gsx,	y + 4 *gsy,
					FILL);

	/* draw gleaming GZ_COL3, most likely red, eye */
	GP142_triangleXY( GZ_COL3,
					x + 6 *gsx, y + 21*gsy,
					x + 4 *gsx, y + 22*gsy,
					x + 5 *gsx,	y + 21*gsy,
					FILL);

}


/*****************************************************************************

     OTHER

*****************************************************************************/


/* Initialize random number generator */
/* [Ignore or delete if your program  */
/* doesn't user random numbers.] */
void initialize_random_numbers() {
	time_t t;			/* current time */
	srand (time(&t));
}



/* Display error message if internal error is detected. */
void something_is_wrong (void) {
	GP142_printfXY (RED, 0, 0, 16, "BAAAADDD NEWS!");
}